/*
 * Generated by gdbus-codegen 2.50.1. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifndef __ORG_GNOME_SESSIONMANAGER_CLIENTPRIVATE_H__
#define __ORG_GNOME_SESSIONMANAGER_CLIENTPRIVATE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager.ClientPrivate */

#define GSM_TYPE_EXPORTED_CLIENT_PRIVATE (gsm_exported_client_private_get_type ())
#define GSM_EXPORTED_CLIENT_PRIVATE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE, GsmExportedClientPrivate))
#define GSM_IS_EXPORTED_CLIENT_PRIVATE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE))
#define GSM_EXPORTED_CLIENT_PRIVATE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE, GsmExportedClientPrivateIface))

struct _GsmExportedClientPrivate;
typedef struct _GsmExportedClientPrivate GsmExportedClientPrivate;
typedef struct _GsmExportedClientPrivateIface GsmExportedClientPrivateIface;

struct _GsmExportedClientPrivateIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_end_session_response) (
    GsmExportedClientPrivate *object,
    GDBusMethodInvocation *invocation,
    gboolean arg_is_ok,
    const gchar *arg_reason);

  void (*cancel_end_session) (
    GsmExportedClientPrivate *object);

  void (*end_session) (
    GsmExportedClientPrivate *object,
    guint arg_flags);

  void (*query_end_session) (
    GsmExportedClientPrivate *object,
    guint arg_flags);

  void (*stop) (
    GsmExportedClientPrivate *object);

};

GType gsm_exported_client_private_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gsm_exported_client_private_interface_info (void);
guint gsm_exported_client_private_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gsm_exported_client_private_complete_end_session_response (
    GsmExportedClientPrivate *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void gsm_exported_client_private_emit_stop (
    GsmExportedClientPrivate *object);

void gsm_exported_client_private_emit_query_end_session (
    GsmExportedClientPrivate *object,
    guint arg_flags);

void gsm_exported_client_private_emit_end_session (
    GsmExportedClientPrivate *object,
    guint arg_flags);

void gsm_exported_client_private_emit_cancel_end_session (
    GsmExportedClientPrivate *object);



/* D-Bus method calls: */
void gsm_exported_client_private_call_end_session_response (
    GsmExportedClientPrivate *proxy,
    gboolean arg_is_ok,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gsm_exported_client_private_call_end_session_response_finish (
    GsmExportedClientPrivate *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gsm_exported_client_private_call_end_session_response_sync (
    GsmExportedClientPrivate *proxy,
    gboolean arg_is_ok,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY (gsm_exported_client_private_proxy_get_type ())
#define GSM_EXPORTED_CLIENT_PRIVATE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY, GsmExportedClientPrivateProxy))
#define GSM_EXPORTED_CLIENT_PRIVATE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY, GsmExportedClientPrivateProxyClass))
#define GSM_EXPORTED_CLIENT_PRIVATE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY, GsmExportedClientPrivateProxyClass))
#define GSM_IS_EXPORTED_CLIENT_PRIVATE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY))
#define GSM_IS_EXPORTED_CLIENT_PRIVATE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_PROXY))

typedef struct _GsmExportedClientPrivateProxy GsmExportedClientPrivateProxy;
typedef struct _GsmExportedClientPrivateProxyClass GsmExportedClientPrivateProxyClass;
typedef struct _GsmExportedClientPrivateProxyPrivate GsmExportedClientPrivateProxyPrivate;

struct _GsmExportedClientPrivateProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GsmExportedClientPrivateProxyPrivate *priv;
};

struct _GsmExportedClientPrivateProxyClass
{
  GDBusProxyClass parent_class;
};

GType gsm_exported_client_private_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GsmExportedClientPrivateProxy, g_object_unref)
#endif

void gsm_exported_client_private_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GsmExportedClientPrivate *gsm_exported_client_private_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GsmExportedClientPrivate *gsm_exported_client_private_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gsm_exported_client_private_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GsmExportedClientPrivate *gsm_exported_client_private_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GsmExportedClientPrivate *gsm_exported_client_private_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON (gsm_exported_client_private_skeleton_get_type ())
#define GSM_EXPORTED_CLIENT_PRIVATE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON, GsmExportedClientPrivateSkeleton))
#define GSM_EXPORTED_CLIENT_PRIVATE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON, GsmExportedClientPrivateSkeletonClass))
#define GSM_EXPORTED_CLIENT_PRIVATE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON, GsmExportedClientPrivateSkeletonClass))
#define GSM_IS_EXPORTED_CLIENT_PRIVATE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON))
#define GSM_IS_EXPORTED_CLIENT_PRIVATE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSM_TYPE_EXPORTED_CLIENT_PRIVATE_SKELETON))

typedef struct _GsmExportedClientPrivateSkeleton GsmExportedClientPrivateSkeleton;
typedef struct _GsmExportedClientPrivateSkeletonClass GsmExportedClientPrivateSkeletonClass;
typedef struct _GsmExportedClientPrivateSkeletonPrivate GsmExportedClientPrivateSkeletonPrivate;

struct _GsmExportedClientPrivateSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GsmExportedClientPrivateSkeletonPrivate *priv;
};

struct _GsmExportedClientPrivateSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gsm_exported_client_private_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GsmExportedClientPrivateSkeleton, g_object_unref)
#endif

GsmExportedClientPrivate *gsm_exported_client_private_skeleton_new (void);


G_END_DECLS

#endif /* __ORG_GNOME_SESSIONMANAGER_CLIENTPRIVATE_H__ */
