/*
 * Generated by gdbus-codegen 2.50.1. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifndef __ORG_GNOME_SESSIONMANAGER_PRESENCE_H__
#define __ORG_GNOME_SESSIONMANAGER_PRESENCE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager.Presence */

#define GSM_TYPE_EXPORTED_PRESENCE (gsm_exported_presence_get_type ())
#define GSM_EXPORTED_PRESENCE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_PRESENCE, GsmExportedPresence))
#define GSM_IS_EXPORTED_PRESENCE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_PRESENCE))
#define GSM_EXPORTED_PRESENCE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GSM_TYPE_EXPORTED_PRESENCE, GsmExportedPresenceIface))

struct _GsmExportedPresence;
typedef struct _GsmExportedPresence GsmExportedPresence;
typedef struct _GsmExportedPresenceIface GsmExportedPresenceIface;

struct _GsmExportedPresenceIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_set_status) (
    GsmExportedPresence *object,
    GDBusMethodInvocation *invocation,
    guint arg_status);

  gboolean (*handle_set_status_text) (
    GsmExportedPresence *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_status_text);

  guint  (*get_status) (GsmExportedPresence *object);

  const gchar * (*get_status_text) (GsmExportedPresence *object);

  void (*status_changed) (
    GsmExportedPresence *object,
    guint arg_status);

  void (*status_text_changed) (
    GsmExportedPresence *object,
    const gchar *arg_status_text);

};

GType gsm_exported_presence_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gsm_exported_presence_interface_info (void);
guint gsm_exported_presence_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gsm_exported_presence_complete_set_status (
    GsmExportedPresence *object,
    GDBusMethodInvocation *invocation);

void gsm_exported_presence_complete_set_status_text (
    GsmExportedPresence *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void gsm_exported_presence_emit_status_changed (
    GsmExportedPresence *object,
    guint arg_status);

void gsm_exported_presence_emit_status_text_changed (
    GsmExportedPresence *object,
    const gchar *arg_status_text);



/* D-Bus method calls: */
void gsm_exported_presence_call_set_status (
    GsmExportedPresence *proxy,
    guint arg_status,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gsm_exported_presence_call_set_status_finish (
    GsmExportedPresence *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gsm_exported_presence_call_set_status_sync (
    GsmExportedPresence *proxy,
    guint arg_status,
    GCancellable *cancellable,
    GError **error);

void gsm_exported_presence_call_set_status_text (
    GsmExportedPresence *proxy,
    const gchar *arg_status_text,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gsm_exported_presence_call_set_status_text_finish (
    GsmExportedPresence *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gsm_exported_presence_call_set_status_text_sync (
    GsmExportedPresence *proxy,
    const gchar *arg_status_text,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint gsm_exported_presence_get_status (GsmExportedPresence *object);
void gsm_exported_presence_set_status (GsmExportedPresence *object, guint value);

const gchar *gsm_exported_presence_get_status_text (GsmExportedPresence *object);
gchar *gsm_exported_presence_dup_status_text (GsmExportedPresence *object);
void gsm_exported_presence_set_status_text (GsmExportedPresence *object, const gchar *value);


/* ---- */

#define GSM_TYPE_EXPORTED_PRESENCE_PROXY (gsm_exported_presence_proxy_get_type ())
#define GSM_EXPORTED_PRESENCE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_PRESENCE_PROXY, GsmExportedPresenceProxy))
#define GSM_EXPORTED_PRESENCE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GSM_TYPE_EXPORTED_PRESENCE_PROXY, GsmExportedPresenceProxyClass))
#define GSM_EXPORTED_PRESENCE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GSM_TYPE_EXPORTED_PRESENCE_PROXY, GsmExportedPresenceProxyClass))
#define GSM_IS_EXPORTED_PRESENCE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_PRESENCE_PROXY))
#define GSM_IS_EXPORTED_PRESENCE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSM_TYPE_EXPORTED_PRESENCE_PROXY))

typedef struct _GsmExportedPresenceProxy GsmExportedPresenceProxy;
typedef struct _GsmExportedPresenceProxyClass GsmExportedPresenceProxyClass;
typedef struct _GsmExportedPresenceProxyPrivate GsmExportedPresenceProxyPrivate;

struct _GsmExportedPresenceProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GsmExportedPresenceProxyPrivate *priv;
};

struct _GsmExportedPresenceProxyClass
{
  GDBusProxyClass parent_class;
};

GType gsm_exported_presence_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GsmExportedPresenceProxy, g_object_unref)
#endif

void gsm_exported_presence_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GsmExportedPresence *gsm_exported_presence_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GsmExportedPresence *gsm_exported_presence_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gsm_exported_presence_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GsmExportedPresence *gsm_exported_presence_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GsmExportedPresence *gsm_exported_presence_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GSM_TYPE_EXPORTED_PRESENCE_SKELETON (gsm_exported_presence_skeleton_get_type ())
#define GSM_EXPORTED_PRESENCE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GSM_TYPE_EXPORTED_PRESENCE_SKELETON, GsmExportedPresenceSkeleton))
#define GSM_EXPORTED_PRESENCE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GSM_TYPE_EXPORTED_PRESENCE_SKELETON, GsmExportedPresenceSkeletonClass))
#define GSM_EXPORTED_PRESENCE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GSM_TYPE_EXPORTED_PRESENCE_SKELETON, GsmExportedPresenceSkeletonClass))
#define GSM_IS_EXPORTED_PRESENCE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSM_TYPE_EXPORTED_PRESENCE_SKELETON))
#define GSM_IS_EXPORTED_PRESENCE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSM_TYPE_EXPORTED_PRESENCE_SKELETON))

typedef struct _GsmExportedPresenceSkeleton GsmExportedPresenceSkeleton;
typedef struct _GsmExportedPresenceSkeletonClass GsmExportedPresenceSkeletonClass;
typedef struct _GsmExportedPresenceSkeletonPrivate GsmExportedPresenceSkeletonPrivate;

struct _GsmExportedPresenceSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GsmExportedPresenceSkeletonPrivate *priv;
};

struct _GsmExportedPresenceSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gsm_exported_presence_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GsmExportedPresenceSkeleton, g_object_unref)
#endif

GsmExportedPresence *gsm_exported_presence_skeleton_new (void);


G_END_DECLS

#endif /* __ORG_GNOME_SESSIONMANAGER_PRESENCE_H__ */
