/*
 * Generated by gdbus-codegen 2.50.1. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "org.gnome.SessionManager.Inhibitor.h"

#include <string.h>
#ifdef G_OS_UNIX
#  include <gio/gunixfdlist.h>
#endif

typedef struct
{
  GDBusArgInfo parent_struct;
  gboolean use_gvariant;
} _ExtendedGDBusArgInfo;

typedef struct
{
  GDBusMethodInfo parent_struct;
  const gchar *signal_name;
  gboolean pass_fdlist;
} _ExtendedGDBusMethodInfo;

typedef struct
{
  GDBusSignalInfo parent_struct;
  const gchar *signal_name;
} _ExtendedGDBusSignalInfo;

typedef struct
{
  GDBusPropertyInfo parent_struct;
  const gchar *hyphen_name;
  gboolean use_gvariant;
} _ExtendedGDBusPropertyInfo;

typedef struct
{
  GDBusInterfaceInfo parent_struct;
  const gchar *hyphen_name;
} _ExtendedGDBusInterfaceInfo;

typedef struct
{
  const _ExtendedGDBusPropertyInfo *info;
  guint prop_id;
  GValue orig_value; /* the value before the change */
} ChangedProperty;

static void
_changed_property_free (ChangedProperty *data)
{
  g_value_unset (&data->orig_value);
  g_free (data);
}

static gboolean
_g_strv_equal0 (gchar **a, gchar **b)
{
  gboolean ret = FALSE;
  guint n;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  if (g_strv_length (a) != g_strv_length (b))
    goto out;
  for (n = 0; a[n] != NULL; n++)
    if (g_strcmp0 (a[n], b[n]) != 0)
      goto out;
  ret = TRUE;
out:
  return ret;
}

static gboolean
_g_variant_equal0 (GVariant *a, GVariant *b)
{
  gboolean ret = FALSE;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  ret = g_variant_equal (a, b);
out:
  return ret;
}

G_GNUC_UNUSED static gboolean
_g_value_equal (const GValue *a, const GValue *b)
{
  gboolean ret = FALSE;
  g_assert (G_VALUE_TYPE (a) == G_VALUE_TYPE (b));
  switch (G_VALUE_TYPE (a))
    {
      case G_TYPE_BOOLEAN:
        ret = (g_value_get_boolean (a) == g_value_get_boolean (b));
        break;
      case G_TYPE_UCHAR:
        ret = (g_value_get_uchar (a) == g_value_get_uchar (b));
        break;
      case G_TYPE_INT:
        ret = (g_value_get_int (a) == g_value_get_int (b));
        break;
      case G_TYPE_UINT:
        ret = (g_value_get_uint (a) == g_value_get_uint (b));
        break;
      case G_TYPE_INT64:
        ret = (g_value_get_int64 (a) == g_value_get_int64 (b));
        break;
      case G_TYPE_UINT64:
        ret = (g_value_get_uint64 (a) == g_value_get_uint64 (b));
        break;
      case G_TYPE_DOUBLE:
        {
          /* Avoid -Wfloat-equal warnings by doing a direct bit compare */
          gdouble da = g_value_get_double (a);
          gdouble db = g_value_get_double (b);
          ret = memcmp (&da, &db, sizeof (gdouble)) == 0;
        }
        break;
      case G_TYPE_STRING:
        ret = (g_strcmp0 (g_value_get_string (a), g_value_get_string (b)) == 0);
        break;
      case G_TYPE_VARIANT:
        ret = _g_variant_equal0 (g_value_get_variant (a), g_value_get_variant (b));
        break;
      default:
        if (G_VALUE_TYPE (a) == G_TYPE_STRV)
          ret = _g_strv_equal0 (g_value_get_boxed (a), g_value_get_boxed (b));
        else
          g_critical ("_g_value_equal() does not handle type %s", g_type_name (G_VALUE_TYPE (a)));
        break;
    }
  return ret;
}

/* ------------------------------------------------------------------------
 * Code for interface org.gnome.SessionManager.Inhibitor
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:GsmExportedInhibitor
 * @title: GsmExportedInhibitor
 * @short_description: Generated C code for the org.gnome.SessionManager.Inhibitor D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link> D-Bus interface in C.
 */

/* ---- Introspection data for org.gnome.SessionManager.Inhibitor ---- */

static const _ExtendedGDBusArgInfo _gsm_exported_inhibitor_method_info_get_app_id_OUT_ARG_app_id =
{
  {
    -1,
    (gchar *) "app_id",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo * const _gsm_exported_inhibitor_method_info_get_app_id_OUT_ARG_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_app_id_OUT_ARG_app_id,
  NULL
};

static const _ExtendedGDBusMethodInfo _gsm_exported_inhibitor_method_info_get_app_id =
{
  {
    -1,
    (gchar *) "GetAppId",
    NULL,
    (GDBusArgInfo **) &_gsm_exported_inhibitor_method_info_get_app_id_OUT_ARG_pointers,
    NULL
  },
  "handle-get-app-id",
  FALSE
};

static const _ExtendedGDBusArgInfo _gsm_exported_inhibitor_method_info_get_client_id_OUT_ARG_client_id =
{
  {
    -1,
    (gchar *) "client_id",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo * const _gsm_exported_inhibitor_method_info_get_client_id_OUT_ARG_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_client_id_OUT_ARG_client_id,
  NULL
};

static const _ExtendedGDBusMethodInfo _gsm_exported_inhibitor_method_info_get_client_id =
{
  {
    -1,
    (gchar *) "GetClientId",
    NULL,
    (GDBusArgInfo **) &_gsm_exported_inhibitor_method_info_get_client_id_OUT_ARG_pointers,
    NULL
  },
  "handle-get-client-id",
  FALSE
};

static const _ExtendedGDBusArgInfo _gsm_exported_inhibitor_method_info_get_reason_OUT_ARG_reason =
{
  {
    -1,
    (gchar *) "reason",
    (gchar *) "s",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo * const _gsm_exported_inhibitor_method_info_get_reason_OUT_ARG_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_reason_OUT_ARG_reason,
  NULL
};

static const _ExtendedGDBusMethodInfo _gsm_exported_inhibitor_method_info_get_reason =
{
  {
    -1,
    (gchar *) "GetReason",
    NULL,
    (GDBusArgInfo **) &_gsm_exported_inhibitor_method_info_get_reason_OUT_ARG_pointers,
    NULL
  },
  "handle-get-reason",
  FALSE
};

static const _ExtendedGDBusArgInfo _gsm_exported_inhibitor_method_info_get_flags_OUT_ARG_flags =
{
  {
    -1,
    (gchar *) "flags",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo * const _gsm_exported_inhibitor_method_info_get_flags_OUT_ARG_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_flags_OUT_ARG_flags,
  NULL
};

static const _ExtendedGDBusMethodInfo _gsm_exported_inhibitor_method_info_get_flags =
{
  {
    -1,
    (gchar *) "GetFlags",
    NULL,
    (GDBusArgInfo **) &_gsm_exported_inhibitor_method_info_get_flags_OUT_ARG_pointers,
    NULL
  },
  "handle-get-flags",
  FALSE
};

static const _ExtendedGDBusArgInfo _gsm_exported_inhibitor_method_info_get_toplevel_xid_OUT_ARG_xid =
{
  {
    -1,
    (gchar *) "xid",
    (gchar *) "u",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo * const _gsm_exported_inhibitor_method_info_get_toplevel_xid_OUT_ARG_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_toplevel_xid_OUT_ARG_xid,
  NULL
};

static const _ExtendedGDBusMethodInfo _gsm_exported_inhibitor_method_info_get_toplevel_xid =
{
  {
    -1,
    (gchar *) "GetToplevelXid",
    NULL,
    (GDBusArgInfo **) &_gsm_exported_inhibitor_method_info_get_toplevel_xid_OUT_ARG_pointers,
    NULL
  },
  "handle-get-toplevel-xid",
  FALSE
};

static const _ExtendedGDBusMethodInfo * const _gsm_exported_inhibitor_method_info_pointers[] =
{
  &_gsm_exported_inhibitor_method_info_get_app_id,
  &_gsm_exported_inhibitor_method_info_get_client_id,
  &_gsm_exported_inhibitor_method_info_get_reason,
  &_gsm_exported_inhibitor_method_info_get_flags,
  &_gsm_exported_inhibitor_method_info_get_toplevel_xid,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _gsm_exported_inhibitor_interface_info =
{
  {
    -1,
    (gchar *) "org.gnome.SessionManager.Inhibitor",
    (GDBusMethodInfo **) &_gsm_exported_inhibitor_method_info_pointers,
    NULL,
    NULL,
    NULL
  },
  "exported-inhibitor",
};


/**
 * gsm_exported_inhibitor_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
gsm_exported_inhibitor_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct;
}

/**
 * gsm_exported_inhibitor_override_properties:
 * @klass: The class structure for a #GObject<!-- -->-derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #GsmExportedInhibitor interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
gsm_exported_inhibitor_override_properties (GObjectClass *klass, guint property_id_begin)
{
  return property_id_begin - 1;
}



/**
 * GsmExportedInhibitor:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link>.
 */

/**
 * GsmExportedInhibitorIface:
 * @parent_iface: The parent interface.
 * @handle_get_app_id: Handler for the #GsmExportedInhibitor::handle-get-app-id signal.
 * @handle_get_client_id: Handler for the #GsmExportedInhibitor::handle-get-client-id signal.
 * @handle_get_flags: Handler for the #GsmExportedInhibitor::handle-get-flags signal.
 * @handle_get_reason: Handler for the #GsmExportedInhibitor::handle-get-reason signal.
 * @handle_get_toplevel_xid: Handler for the #GsmExportedInhibitor::handle-get-toplevel-xid signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link>.
 */

typedef GsmExportedInhibitorIface GsmExportedInhibitorInterface;
G_DEFINE_INTERFACE (GsmExportedInhibitor, gsm_exported_inhibitor, G_TYPE_OBJECT);

static void
gsm_exported_inhibitor_default_init (GsmExportedInhibitorIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * GsmExportedInhibitor::handle-get-app-id:
   * @object: A #GsmExportedInhibitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetAppId">GetAppId()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call gsm_exported_inhibitor_complete_get_app_id() or e.g. g_dbus_method_invocation_return_error() on it) and no order signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %TRUE if the invocation was handled, %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-app-id",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (GsmExportedInhibitorIface, handle_get_app_id),
    g_signal_accumulator_true_handled,
    NULL,
    g_cclosure_marshal_generic,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * GsmExportedInhibitor::handle-get-client-id:
   * @object: A #GsmExportedInhibitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetClientId">GetClientId()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call gsm_exported_inhibitor_complete_get_client_id() or e.g. g_dbus_method_invocation_return_error() on it) and no order signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %TRUE if the invocation was handled, %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-client-id",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (GsmExportedInhibitorIface, handle_get_client_id),
    g_signal_accumulator_true_handled,
    NULL,
    g_cclosure_marshal_generic,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * GsmExportedInhibitor::handle-get-reason:
   * @object: A #GsmExportedInhibitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetReason">GetReason()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call gsm_exported_inhibitor_complete_get_reason() or e.g. g_dbus_method_invocation_return_error() on it) and no order signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %TRUE if the invocation was handled, %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-reason",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (GsmExportedInhibitorIface, handle_get_reason),
    g_signal_accumulator_true_handled,
    NULL,
    g_cclosure_marshal_generic,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * GsmExportedInhibitor::handle-get-flags:
   * @object: A #GsmExportedInhibitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetFlags">GetFlags()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call gsm_exported_inhibitor_complete_get_flags() or e.g. g_dbus_method_invocation_return_error() on it) and no order signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %TRUE if the invocation was handled, %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-flags",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (GsmExportedInhibitorIface, handle_get_flags),
    g_signal_accumulator_true_handled,
    NULL,
    g_cclosure_marshal_generic,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

  /**
   * GsmExportedInhibitor::handle-get-toplevel-xid:
   * @object: A #GsmExportedInhibitor.
   * @invocation: A #GDBusMethodInvocation.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetToplevelXid">GetToplevelXid()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call gsm_exported_inhibitor_complete_get_toplevel_xid() or e.g. g_dbus_method_invocation_return_error() on it) and no order signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %TRUE if the invocation was handled, %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-get-toplevel-xid",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (GsmExportedInhibitorIface, handle_get_toplevel_xid),
    g_signal_accumulator_true_handled,
    NULL,
    g_cclosure_marshal_generic,
    G_TYPE_BOOLEAN,
    1,
    G_TYPE_DBUS_METHOD_INVOCATION);

}

/**
 * gsm_exported_inhibitor_call_get_app_id:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetAppId">GetAppId()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_call_get_app_id_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_call_get_app_id_sync() for the synchronous, blocking version of this method.
 */
void
gsm_exported_inhibitor_call_get_app_id (
    GsmExportedInhibitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetAppId",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * gsm_exported_inhibitor_call_get_app_id_finish:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_app_id: (out): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_call_get_app_id().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with gsm_exported_inhibitor_call_get_app_id().
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_app_id_finish (
    GsmExportedInhibitor *proxy,
    gchar **out_app_id,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_app_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_app_id_sync:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_app_id: (out): Return location for return parameter or %NULL to ignore.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetAppId">GetAppId()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_call_get_app_id() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_app_id_sync (
    GsmExportedInhibitor *proxy,
    gchar **out_app_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetAppId",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_app_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_client_id:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetClientId">GetClientId()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_call_get_client_id_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_call_get_client_id_sync() for the synchronous, blocking version of this method.
 */
void
gsm_exported_inhibitor_call_get_client_id (
    GsmExportedInhibitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetClientId",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * gsm_exported_inhibitor_call_get_client_id_finish:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_client_id: (out): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_call_get_client_id().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with gsm_exported_inhibitor_call_get_client_id().
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_client_id_finish (
    GsmExportedInhibitor *proxy,
    gchar **out_client_id,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_client_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_client_id_sync:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_client_id: (out): Return location for return parameter or %NULL to ignore.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetClientId">GetClientId()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_call_get_client_id() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_client_id_sync (
    GsmExportedInhibitor *proxy,
    gchar **out_client_id,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetClientId",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(o)",
                 out_client_id);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_reason:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetReason">GetReason()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_call_get_reason_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_call_get_reason_sync() for the synchronous, blocking version of this method.
 */
void
gsm_exported_inhibitor_call_get_reason (
    GsmExportedInhibitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetReason",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * gsm_exported_inhibitor_call_get_reason_finish:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_reason: (out): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_call_get_reason().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with gsm_exported_inhibitor_call_get_reason().
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_reason_finish (
    GsmExportedInhibitor *proxy,
    gchar **out_reason,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_reason);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_reason_sync:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_reason: (out): Return location for return parameter or %NULL to ignore.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetReason">GetReason()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_call_get_reason() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_reason_sync (
    GsmExportedInhibitor *proxy,
    gchar **out_reason,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetReason",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(s)",
                 out_reason);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_flags:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetFlags">GetFlags()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_call_get_flags_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_call_get_flags_sync() for the synchronous, blocking version of this method.
 */
void
gsm_exported_inhibitor_call_get_flags (
    GsmExportedInhibitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetFlags",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * gsm_exported_inhibitor_call_get_flags_finish:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_flags: (out): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_call_get_flags().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with gsm_exported_inhibitor_call_get_flags().
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_flags_finish (
    GsmExportedInhibitor *proxy,
    guint *out_flags,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_flags);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_flags_sync:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_flags: (out): Return location for return parameter or %NULL to ignore.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetFlags">GetFlags()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_call_get_flags() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_flags_sync (
    GsmExportedInhibitor *proxy,
    guint *out_flags,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetFlags",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_flags);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_toplevel_xid:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetToplevelXid">GetToplevelXid()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_call_get_toplevel_xid_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_call_get_toplevel_xid_sync() for the synchronous, blocking version of this method.
 */
void
gsm_exported_inhibitor_call_get_toplevel_xid (
    GsmExportedInhibitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "GetToplevelXid",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * gsm_exported_inhibitor_call_get_toplevel_xid_finish:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_xid: (out): Return location for return parameter or %NULL to ignore.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_call_get_toplevel_xid().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with gsm_exported_inhibitor_call_get_toplevel_xid().
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_toplevel_xid_finish (
    GsmExportedInhibitor *proxy,
    guint *out_xid,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_xid);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_call_get_toplevel_xid_sync:
 * @proxy: A #GsmExportedInhibitorProxy.
 * @out_xid: (out): Return location for return parameter or %NULL to ignore.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetToplevelXid">GetToplevelXid()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_call_get_toplevel_xid() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeded, %FALSE if @error is set.
 */
gboolean
gsm_exported_inhibitor_call_get_toplevel_xid_sync (
    GsmExportedInhibitor *proxy,
    guint *out_xid,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "GetToplevelXid",
    g_variant_new ("()"),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "(u)",
                 out_xid);
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * gsm_exported_inhibitor_complete_get_app_id:
 * @object: A #GsmExportedInhibitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @app_id: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetAppId">GetAppId()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
gsm_exported_inhibitor_complete_get_app_id (
    GsmExportedInhibitor *object,
    GDBusMethodInvocation *invocation,
    const gchar *app_id)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   app_id));
}

/**
 * gsm_exported_inhibitor_complete_get_client_id:
 * @object: A #GsmExportedInhibitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @client_id: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetClientId">GetClientId()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
gsm_exported_inhibitor_complete_get_client_id (
    GsmExportedInhibitor *object,
    GDBusMethodInvocation *invocation,
    const gchar *client_id)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(o)",
                   client_id));
}

/**
 * gsm_exported_inhibitor_complete_get_reason:
 * @object: A #GsmExportedInhibitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @reason: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetReason">GetReason()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
gsm_exported_inhibitor_complete_get_reason (
    GsmExportedInhibitor *object,
    GDBusMethodInvocation *invocation,
    const gchar *reason)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(s)",
                   reason));
}

/**
 * gsm_exported_inhibitor_complete_get_flags:
 * @object: A #GsmExportedInhibitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @flags: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetFlags">GetFlags()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
gsm_exported_inhibitor_complete_get_flags (
    GsmExportedInhibitor *object,
    GDBusMethodInvocation *invocation,
    guint flags)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   flags));
}

/**
 * gsm_exported_inhibitor_complete_get_toplevel_xid:
 * @object: A #GsmExportedInhibitor.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 * @xid: Parameter to return.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-gnome-SessionManager-Inhibitor.GetToplevelXid">GetToplevelXid()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
gsm_exported_inhibitor_complete_get_toplevel_xid (
    GsmExportedInhibitor *object,
    GDBusMethodInvocation *invocation,
    guint xid)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("(u)",
                   xid));
}

/* ------------------------------------------------------------------------ */

/**
 * GsmExportedInhibitorProxy:
 *
 * The #GsmExportedInhibitorProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * GsmExportedInhibitorProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #GsmExportedInhibitorProxy.
 */

struct _GsmExportedInhibitorProxyPrivate
{
  GData *qdata;
};

static void gsm_exported_inhibitor_proxy_iface_init (GsmExportedInhibitorIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (GsmExportedInhibitorProxy, gsm_exported_inhibitor_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (GsmExportedInhibitorProxy)
                         G_IMPLEMENT_INTERFACE (GSM_TYPE_EXPORTED_INHIBITOR, gsm_exported_inhibitor_proxy_iface_init));

#else
G_DEFINE_TYPE_WITH_CODE (GsmExportedInhibitorProxy, gsm_exported_inhibitor_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (GSM_TYPE_EXPORTED_INHIBITOR, gsm_exported_inhibitor_proxy_iface_init));

#endif
static void
gsm_exported_inhibitor_proxy_finalize (GObject *object)
{
  GsmExportedInhibitorProxy *proxy = GSM_EXPORTED_INHIBITOR_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (gsm_exported_inhibitor_proxy_parent_class)->finalize (object);
}

static void
gsm_exported_inhibitor_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
gsm_exported_inhibitor_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
}

static void
gsm_exported_inhibitor_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  guint num_params;
  guint n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], GSM_TYPE_EXPORTED_INHIBITOR);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, GSM_TYPE_EXPORTED_INHIBITOR);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
gsm_exported_inhibitor_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  GsmExportedInhibitorProxy *proxy = GSM_EXPORTED_INHIBITOR_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static void
gsm_exported_inhibitor_proxy_init (GsmExportedInhibitorProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = gsm_exported_inhibitor_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, GSM_TYPE_EXPORTED_INHIBITOR_PROXY, GsmExportedInhibitorProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), gsm_exported_inhibitor_interface_info ());
}

static void
gsm_exported_inhibitor_proxy_class_init (GsmExportedInhibitorProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = gsm_exported_inhibitor_proxy_finalize;
  gobject_class->get_property = gsm_exported_inhibitor_proxy_get_property;
  gobject_class->set_property = gsm_exported_inhibitor_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = gsm_exported_inhibitor_proxy_g_signal;
  proxy_class->g_properties_changed = gsm_exported_inhibitor_proxy_g_properties_changed;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (GsmExportedInhibitorProxyPrivate));
#endif
}

static void
gsm_exported_inhibitor_proxy_iface_init (GsmExportedInhibitorIface *iface)
{
}

/**
 * gsm_exported_inhibitor_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (allow-none): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_proxy_new_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
gsm_exported_inhibitor_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (GSM_TYPE_EXPORTED_INHIBITOR_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager.Inhibitor", NULL);
}

/**
 * gsm_exported_inhibitor_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with gsm_exported_inhibitor_proxy_new().
 *
 * Returns: (transfer full) (type GsmExportedInhibitorProxy): The constructed proxy object or %NULL if @error is set.
 */
GsmExportedInhibitor *
gsm_exported_inhibitor_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return GSM_EXPORTED_INHIBITOR (ret);
  else
    return NULL;
}

/**
 * gsm_exported_inhibitor_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (allow-none): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type GsmExportedInhibitorProxy): The constructed proxy object or %NULL if @error is set.
 */
GsmExportedInhibitor *
gsm_exported_inhibitor_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (GSM_TYPE_EXPORTED_INHIBITOR_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager.Inhibitor", NULL);
  if (ret != NULL)
    return GSM_EXPORTED_INHIBITOR (ret);
  else
    return NULL;
}


/**
 * gsm_exported_inhibitor_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like gsm_exported_inhibitor_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
 * You can then call gsm_exported_inhibitor_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See gsm_exported_inhibitor_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
gsm_exported_inhibitor_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (GSM_TYPE_EXPORTED_INHIBITOR_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager.Inhibitor", NULL);
}

/**
 * gsm_exported_inhibitor_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to gsm_exported_inhibitor_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with gsm_exported_inhibitor_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type GsmExportedInhibitorProxy): The constructed proxy object or %NULL if @error is set.
 */
GsmExportedInhibitor *
gsm_exported_inhibitor_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return GSM_EXPORTED_INHIBITOR (ret);
  else
    return NULL;
}

/**
 * gsm_exported_inhibitor_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (allow-none): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like gsm_exported_inhibitor_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See gsm_exported_inhibitor_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type GsmExportedInhibitorProxy): The constructed proxy object or %NULL if @error is set.
 */
GsmExportedInhibitor *
gsm_exported_inhibitor_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (GSM_TYPE_EXPORTED_INHIBITOR_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "org.gnome.SessionManager.Inhibitor", NULL);
  if (ret != NULL)
    return GSM_EXPORTED_INHIBITOR (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * GsmExportedInhibitorSkeleton:
 *
 * The #GsmExportedInhibitorSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * GsmExportedInhibitorSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #GsmExportedInhibitorSkeleton.
 */

struct _GsmExportedInhibitorSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_gsm_exported_inhibitor_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  GsmExportedInhibitorSkeleton *skeleton = GSM_EXPORTED_INHIBITOR_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  guint num_params;
  guint num_extra;
  guint n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], GSM_TYPE_EXPORTED_INHIBITOR);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, GSM_TYPE_EXPORTED_INHIBITOR);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_gsm_exported_inhibitor_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  GsmExportedInhibitorSkeleton *skeleton = GSM_EXPORTED_INHIBITOR_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_gsm_exported_inhibitor_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  GsmExportedInhibitorSkeleton *skeleton = GSM_EXPORTED_INHIBITOR_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_gsm_exported_inhibitor_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _gsm_exported_inhibitor_skeleton_vtable =
{
  _gsm_exported_inhibitor_skeleton_handle_method_call,
  _gsm_exported_inhibitor_skeleton_handle_get_property,
  _gsm_exported_inhibitor_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
gsm_exported_inhibitor_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return gsm_exported_inhibitor_interface_info ();
}

static GDBusInterfaceVTable *
gsm_exported_inhibitor_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_gsm_exported_inhibitor_skeleton_vtable;
}

static GVariant *
gsm_exported_inhibitor_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  GsmExportedInhibitorSkeleton *skeleton = GSM_EXPORTED_INHIBITOR_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  if (_gsm_exported_inhibitor_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _gsm_exported_inhibitor_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _gsm_exported_inhibitor_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _gsm_exported_inhibitor_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "org.gnome.SessionManager.Inhibitor", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static void
gsm_exported_inhibitor_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
}

static void gsm_exported_inhibitor_skeleton_iface_init (GsmExportedInhibitorIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (GsmExportedInhibitorSkeleton, gsm_exported_inhibitor_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (GsmExportedInhibitorSkeleton)
                         G_IMPLEMENT_INTERFACE (GSM_TYPE_EXPORTED_INHIBITOR, gsm_exported_inhibitor_skeleton_iface_init));

#else
G_DEFINE_TYPE_WITH_CODE (GsmExportedInhibitorSkeleton, gsm_exported_inhibitor_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (GSM_TYPE_EXPORTED_INHIBITOR, gsm_exported_inhibitor_skeleton_iface_init));

#endif
static void
gsm_exported_inhibitor_skeleton_finalize (GObject *object)
{
  GsmExportedInhibitorSkeleton *skeleton = GSM_EXPORTED_INHIBITOR_SKELETON (object);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (gsm_exported_inhibitor_skeleton_parent_class)->finalize (object);
}

static void
gsm_exported_inhibitor_skeleton_init (GsmExportedInhibitorSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = gsm_exported_inhibitor_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, GSM_TYPE_EXPORTED_INHIBITOR_SKELETON, GsmExportedInhibitorSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
}

static void
gsm_exported_inhibitor_skeleton_class_init (GsmExportedInhibitorSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = gsm_exported_inhibitor_skeleton_finalize;

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = gsm_exported_inhibitor_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = gsm_exported_inhibitor_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = gsm_exported_inhibitor_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = gsm_exported_inhibitor_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (GsmExportedInhibitorSkeletonPrivate));
#endif
}

static void
gsm_exported_inhibitor_skeleton_iface_init (GsmExportedInhibitorIface *iface)
{
}

/**
 * gsm_exported_inhibitor_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-org-gnome-SessionManager-Inhibitor.top_of_page">org.gnome.SessionManager.Inhibitor</link>.
 *
 * Returns: (transfer full) (type GsmExportedInhibitorSkeleton): The skeleton object.
 */
GsmExportedInhibitor *
gsm_exported_inhibitor_skeleton_new (void)
{
  return GSM_EXPORTED_INHIBITOR (g_object_new (GSM_TYPE_EXPORTED_INHIBITOR_SKELETON, NULL));
}

